/**
 *
 * Driver.java
 *
 * Copyright 1998, Junus Junarto D
 *                 (junus@its.ac.id)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * @version 0.1.0 1-MARCH-1998
 *
 * The Java SQL framework allows for multiple database drivers.  Each
 * driver should supply a class that implements the Driver interface
 *
 * The DriverManager will try to load as many drivers as it can find and then
 * for any given connection request, it will ask each driver in turn to try
 * to connect to the target URL.
 *
 * It is strongly recommended that each Driver class should be small and
 * standalone so that the Driver class can be loaded and queried without
 * bringing in vast quantities of supporting code.
 *
 * When a Driver class is loaded, it should create an instance of itself and
 * register it with the DriverManager.  This means that a user can load and
 * register a driver by doing Class.forName("foo.bah.Driver")
 *
 * @see iddb.Connection
 * @see java.sql.Driver
 */
package iddb;

import java.sql.*;
import java.util.*;

public class IDDBLocalConnection
{
    java.sql.Connection db;

    public IDDBLocalConnection(java.sql.Connection db)
    {
        this.db=db;
    }

  public IDDBLocalStatement createStatement() throws SQLException
  {
    return new IDDBLocalStatement (db.createStatement());
  }

}
