/**
 *
 * Driver.java
 *
 * Copyright 1998, Junus Junarto D
 *                 (junus@its.ac.id)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * @version 0.1.0 1-MARCH-1998
 *
 * The Java SQL framework allows for multiple database drivers.  Each
 * driver should supply a class that implements the Driver interface
 *
 * The DriverManager will try to load as many drivers as it can find and then
 * for any given connection request, it will ask each driver in turn to try
 * to connect to the target URL.
 *
 * It is strongly recommended that each Driver class should be small and
 * standalone so that the Driver class can be loaded and queried without
 * bringing in vast quantities of supporting code.
 *
 * When a Driver class is loaded, it should create an instance of itself and
 * register it with the DriverManager.  This means that a user can load and
 * register a driver by doing Class.forName("foo.bah.Driver")
 *
 * @see iddb.Connection
 * @see java.sql.Driver
 */
package iddb;

import java.sql.*;
import java.util.*;

public class IDDBLocalStatement
{
    int SELECT=1;
    int INSERT=2;
    int DELETE=3;
    int UPDATE=4;

    java.sql.Statement s;

    public IDDBLocalStatement(java.sql.Statement s)
    {
        this.s=s;
    }

    public java.sql.ResultSet executeQuery(String str,String user) throws SQLException
    {
        log(str,user);
        return s.executeQuery(str);
    }

    public int executeUpdate(String str,String user) throws SQLException
    {
        log(str,user);
        return s.executeUpdate(str);
    }

    private String fReplaceString(String vpString,String vpA, String vpB)
    {
        String vfTmp=vpString;
        int vfI=0;
    	int vfX = vfTmp.indexOf(vpA,vfI);
    	while(vfX>=0)
        {
        	vfTmp=vfTmp.substring(0,vfX)+vpB+vfTmp.substring(vfX+vpA.length());
            vfI=vfX+vpB.length();
    	    vfX = vfTmp.indexOf(vpA,vfI);
        }
    	return vfTmp;
    }

    public void log(String str, String logUser) throws SQLException
    {
        String logstr;
        String logTable;
        int op;

        str.toLowerCase();

        StringTokenizer st = new StringTokenizer(str);
        if (st.countTokens()>0)
        {
        	String operation = st.nextToken();

        	if(operation.equals("select"))
        	    op=SELECT;
        	else if(operation.equals("insert"))
        	    op=INSERT;
        	else if(operation.equals("delete"))
        	    op=DELETE;
        	else if(operation.equals("update"))
        	    op=UPDATE;
        	else throw new SQLException("\nError query : "+str+"\nQuery not available in IDDB local mode\n");

            if(op>SELECT)
            {
                if(st.countTokens()>=2)
                {
                    if(op==UPDATE)
                    	logTable = st.nextToken();
                	else
                	{
                    	st.nextToken();
                    	logTable = st.nextToken();
                	}
                	logstr="";
                	try
                	{
                        java.sql.ResultSet vfRS =s.executeQuery("select icounter, isiteid from iddbtbcount"+
                        " where itbname = '"+logTable+"' and isiteid <> '*'");
                        vfRS.next();
                        int vfTbCount=vfRS.getInt(1);
                        String siteid=vfRS.getString(2).trim();

                    	logstr = "insert into log"+logTable+" values ('"+(new java.util.Date(System.currentTimeMillis())).toString()+"', "+vfTbCount+" ,'"+fReplaceString(str,"'","''")+"' ,"+op+" ,'"+logUser+"', 'l"+siteid+"')";
//                        System.out.println(logstr);
                        s.executeUpdate(logstr);

                        s.executeUpdate("update iddbtbcount set icounter = "+ ++vfTbCount +
                        " where itbname = '"+logTable+"'");
                    } catch (SQLException e) {
                        System.out.println("Error : logTable not found !, use IDDBMaster to create logTable : "+logTable+" !");

/*                        String strCreate="create table log"+logTable+" (idate char(30),icounter integer, ioperation char(200), itype integer,iuser char(20), isiteid char(20))";
                        System.out.println(strCreate);
                        s.executeUpdate(strCreate);
                        System.out.println(logstr);
                        s.executeUpdate(logstr);
*/                    }
                }
                else throw new SQLException("No table name");
            }
        }
    }
}
