import java.sql.*;
import java.util.*;

class OptimizerScheduler
{
    private ErrorManager vcError;
	private int vcDebugLevel;

    private Statement vcStatement;
    private Executor vcExecutor;
	private int vcOptimizerType;

    public OptimizerScheduler(Statement vpStatement, int vpOptimizerType, ErrorManager vpError, int vpDebugLevel)
    {
        vcError=vpError;
        vcDebugLevel=vpDebugLevel;
        vcStatement=vpStatement;
        vcOptimizerType=vpOptimizerType;
    }

    public void fInsertExecutor(Executor vpExecutor)
    {
        vcExecutor=vpExecutor;
    }

    public synchronized ResultOut executeQuery(parserout_t vpPO) throws SQLException
    {
        Vector ScheduleList=null;
        if((vpPO.command==IDDBConstants.COMMAND_SELECT)&&(vcOptimizerType==IDDBConstants.OPTIMIZER_SEMIJOIN))
                ScheduleList=fSemiJoin(vpPO);
        return vcExecutor.executeQuery(vpPO,ScheduleList);
    }

    public void fExecuteQueryEnd()
    {
        vcExecutor.fExecuteQueryEnd();
    }

    private Vector fSemiJoin(parserout_t vpPO) throws SQLException
    {
        Vector vfScheduleList= new Vector();
        Hashtable vfTableList= new Hashtable();
        Vector vfTableListSort=new Vector();

        fTableList(vpPO, vfTableList,vfTableListSort);

        Vector vfTableDrop = new Vector();
        Vector vfTableDropRA = new Vector();

        for (Enumeration vfEn = vfTableList.elements() ; vfEn.hasMoreElements() ;)
        {
    	    table_t vfTable=(table_t)vfEn.nextElement();

//            ResultSet vfRSField=vcStatement.executeQuery("select iname, itype,ilength from iddbtbfield where itbname='"+vfTable.name+"' order by ino");
            ResultSet vfRSField=vcStatement.executeQuery("select iname, itype,ilength,ino from iddbtbfield where itbname='"+vfTable.name+"' order by ino");
            Hashtable vfFieldList=new Hashtable();
            Vector vfFieldListOrder=new Vector();
            while(vfRSField.next())
            {
                field_t vfField = new field_t();
    	        vfField.name = vfRSField.getString(1).trim();
    	        vfField.type = vfRSField.getInt(2);
    	        vfField.length = vfRSField.getInt(3);
                vfFieldList.put(vfField.name,vfField);
                vfFieldListOrder.addElement(vfField);
    		}
    		if(vpPO.selectWildcard)
    		{
    		    vfTable.field=vfFieldList;
    		    vfTable.fieldorder=vfFieldListOrder;
    		} else
    		{
                for (Enumeration ex = vfTable.field.elements() ; ex.hasMoreElements() ;)
                {
    	            field_t vfField=(field_t)ex.nextElement();
    	            field_t vfFieldx=(field_t) vfFieldList.get(vfField.name);
    	            if(vfFieldx!=null)
    	            {
    	                vfField.type=vfFieldx.type;
    	                vfField.length=vfFieldx.length;
    	            }else
                	{
                		vcError.fPutError(1801,"OptimizerScheduler Error : Field name not found in table: "+vfField.table+"."+vfField.name,true);
//                		Trace(TRACE_OUT,"ProcessQuery()");
                		return null;
                	}
    		    }
    		}

            ResultSet vfRSSite=vcStatement.executeQuery("select isiteid from iddbtbsite where itbname='"+vfTable.name+"'");
            while(vfRSSite.next())
            {
    	        vfTable.site.addElement(vfRSSite.getString(1).trim());
//    	        vfTable.fragment.addElement(vfRSSite.getString(2).trim());
    		}
    	}

        for (int vfTableNo=0;vfTableNo<vfTableListSort.size(); vfTableNo++)
        {
    	    table_t vfTable=(table_t)vfTableListSort.elementAt(vfTableNo);
            String vfTableCreateField="";
            String vfTableCreateJoin="";
            String vfTmpTable="tmp"+vfTable.name;
            String vfTableField="";
            String vfTableFieldName="";
            String vfTablesName="";
            String vfTableCondJoin="";
            String vfBoolOp="";
            int vfI=0;
            for(Enumeration vfEn = vfTable.fieldorder.elements() ; vfEn.hasMoreElements() ;vfI++)
            {
    	        field_t vfField=(field_t)vfEn.nextElement();
                if(!vfTableCreateField.equals(""))
                    vfTableCreateField+=", ";
                if(!vfTableField.equals(""))
                    vfTableField+=", ";
                String vfTmp=null;
                switch (vfField.type)
                {
                    case Types.CHAR:
                        vfTmp=vfField.name+" char("+vfField.length+")";
                        break;
                    case Types.INTEGER:
                        vfTmp=vfField.name+" integer";
                        break;
                    case Types.FLOAT:
                        vfTmp=vfField.name+" float";
                        break;
                }
                if (vfField.condJoin)
                {
                    if(!vfTableCreateJoin.equals(""))
                        vfTableCreateJoin+=", ";
                    vfTableCreateJoin+=vfTmp;
                    if(vfField.cond.hasbeencreatetable)
                    {
                        String vfTName;
                        if(!vfTableCondJoin.equals(""))
                        {
                            switch(vfField.cond.bool)
                            {
                                case IDDBConstants.AND_BOOL:
                                    vfBoolOp=" and ";
                                    break;
                                case IDDBConstants.OR_BOOL:
                                    vfBoolOp=" or ";
                                    break;
                                default :
                                    vfBoolOp=" and ";
                            }

                            vfTableCondJoin=vfBoolOp+vfTableCondJoin;
                        }
                        if(vfField.cond.table.equals(vfTable.name))
                        {
                            vfTName=vfField.cond.value.identVal.seg1;
                            vfTableCondJoin=vfField.cond.table+"."+vfField.cond.name+Tools.fOp(vfField.cond.op)+
                                      "tmp"+vfTName+"."+vfField.cond.value.identVal.seg2+vfTableCondJoin;
                        }
                        else
                        {
                            vfTName=vfField.cond.table;

                            vfTableCondJoin=vfField.cond.value.identVal.seg1+"."+vfField.cond.value.identVal.seg2+
                                        Tools.fOp(vfField.cond.op)+"tmp"+vfTName+"."+vfField.cond.name+vfTableCondJoin;
                        }
                        vfTablesName+=", tmp"+vfTName;
                    }
                    else
                        vfField.cond.hasbeencreatetable=true;
                }
                vfTableCreateField+=vfTmp;
                vfTableField+=vfTable.name+"."+vfField.name;
            }

            // where
            for(Enumeration vfEn = vfTable.condSelect.elements() ; vfEn.hasMoreElements() ;vfI++)
            {
    	        cond_t vfCond=(cond_t)vfEn.nextElement();
                if(!vfTableCondJoin.equals(""))
                {
                    switch(vfCond.bool)
                    {
                        case IDDBConstants.AND_BOOL:
                            vfBoolOp=" and ";
                            break;
                        case IDDBConstants.OR_BOOL:
                            vfBoolOp=" or ";
                            break;
                        default :
                            vfBoolOp=" and ";
                    }
                    vfTableCondJoin+=vfBoolOp;
                }
                vfTableCondJoin+=vfCond.table+"."+vfCond.name+Tools.fOp(vfCond.op)+Tools.fValue(vfCond.value);
            }

            String vfTable1="create table "+vfTmpTable+"("+vfTableCreateField+")";
            String vfTable2="create table "+vfTmpTable+"("+vfTableCreateJoin+")";
            String vfWhere = (vfTableCondJoin.equals("")? "" : " where "+vfTableCondJoin);
            String vfTable3="select distinct "+vfTableField+" from "+vfTable.name+vfTablesName+vfWhere;

            boolean vfLastTable=vfTableNo==(vfTableListSort.size()-1);
            boolean vfAccessRA=!vfLastTable && !vfTableCreateJoin.equals("");

            vfScheduleList.addElement(new schedule_t(false, IDDBConstants.COMMAND_CREATE_TABLE, vfTable1, null, null));
            vfTableDrop.addElement(vfTable.name);

            if(vfAccessRA)
            {
                vfScheduleList.addElement(new schedule_t(true, IDDBConstants.COMMAND_CREATE_TABLE, vfTable2, null, (Vector) vfTable.site.clone()));
                vfTableDropRA.addElement(vfTable.name);
            }

            vfScheduleList.addElement(new schedule_t(true, IDDBConstants.COMMAND_SELECT, vfTable3,null, (Vector) vfTable.site.clone()));

            vfScheduleList.addElement(new schedule_t(vfAccessRA, IDDBConstants.COMMAND_INSERT, vfTmpTable,(Vector) vfTable.fieldorder.clone(), (Vector) vfTable.site.clone()));
        }

        String vfTmpField="";
        String vfTmpTable="";
        String vfTmpCond="";
        String vfTmpOrder="";
        if(vpPO.selectWildcard)
            vfTmpField="*";
        else
        {
            for(Enumeration vfEn = vpPO.field.elements() ; vfEn.hasMoreElements() ;)
            {
                field_t vfField=(field_t)vfEn.nextElement();
                if(!vfTmpField.equals(""))
                    vfTmpField+=", ";
                vfTmpField+="tmp"+vfField.table+"."+vfField.name;
            }
        }
        for(Enumeration vfEn = vpPO.table.elements() ; vfEn.hasMoreElements() ;)
        {
            tname_t vfTable=(tname_t)vfEn.nextElement();
            if(!vfTmpTable.equals(""))
                vfTmpTable=", "+vfTmpTable;
            if(vfTable.cname==null)
                vfTmpTable="tmp"+vfTable.name+vfTmpTable;
            else
                vfTmpTable="tmp"+vfTable.cname+vfTmpTable;
        }
        for(Enumeration vfEn = vpPO.cond.elements() ; vfEn.hasMoreElements() ;)
        {
            cond_t vfCond=(cond_t)vfEn.nextElement();
            String vfBoolOp;
            if(!vfTmpCond.equals(""))
            {
                switch(vfCond.bool)
                {
                    case IDDBConstants.AND_BOOL:
                        vfBoolOp=" and ";
                        break;
                    case IDDBConstants.OR_BOOL:
                        vfBoolOp=" or ";
                        break;
                    default :
                        vfBoolOp=" and ";
                }
                vfTmpCond+=vfBoolOp;
            }
            if(vfCond.value.type==IDDBConstants.IDENT_TYPE)
                vfTmpCond+="tmp"+vfCond.table+"."+vfCond.name+Tools.fOp(vfCond.op)+"tmp"+vfCond.value.identVal.seg1+"."+
                    vfCond.value.identVal.seg2;
            else
                vfTmpCond+="tmp"+vfCond.table+"."+vfCond.name+Tools.fOp(vfCond.op)+Tools.fValue(vfCond.value);
        }
        for(Enumeration vfEn = vpPO.order.elements() ; vfEn.hasMoreElements() ;)
        {
            order_t vfOrder=(order_t)vfEn.nextElement();
            if(!vfTmpOrder.equals(""))
                vfTmpOrder=", "+vfTmpOrder;
            vfTmpOrder="tmp"+vfOrder.table+"."+vfOrder.name+vfTmpOrder;
        }

        String vfTmpSelect="select "+vfTmpField+" from "+vfTmpTable+
        (vfTmpCond.equals("") ? "":" where "+vfTmpCond)+
        (vfTmpOrder.equals("") ? "":" order by "+vfTmpOrder);
        vfScheduleList.addElement(new schedule_t(false, IDDBConstants.COMMAND_SELECT, vfTmpSelect ,null, null));

    	for (Enumeration vfEn = vfTableDrop.elements() ; vfEn.hasMoreElements() ;)
    	{
    	    String vfTable=(String)vfEn.nextElement();
            vfScheduleList.addElement(new schedule_t(false, IDDBConstants.COMMAND_DROP_TABLE, "drop table tmp"+vfTable,null, null));
    	}
    	for (Enumeration vfEn = vfTableDropRA.elements() ; vfEn.hasMoreElements() ;)
    	{
    	    String vfTable=(String)vfEn.nextElement();
            vfScheduleList.addElement(new schedule_t(true, IDDBConstants.COMMAND_DROP_TABLE, "drop table tmp"+vfTable,null, (Vector) ((table_t)vfTableList.get(vfTable)).site.clone()));
    	}
//        String tableRA="              RA*: ";
    	for (Enumeration vfEn = vfScheduleList.elements() ; vfEn.hasMoreElements() ;)
    	{
    	    schedule_t vfSchedule=(schedule_t)vfEn.nextElement();
//    	    if(t.RA)
//    	        System.out.print(tableRA);
            if(vcDebugLevel>=1)
    	        System.out.println(vfSchedule.query);
    	}
    	return vfScheduleList;

    }

    private void fTableList(parserout_t vpPO, Hashtable vpTableList, Vector vpTableListSort)
    {
        if(vcDebugLevel>=1)
            System.out.println("IN processtablelist");
    // buat vfTableList
    	for (Enumeration vfEn = vpPO.table.elements() ; vfEn.hasMoreElements() ;)
    	{
    	    tname_t vfTName=(tname_t)vfEn.nextElement();
    	    if(vfTName.cname==null)
    	        vpTableList.put(vfTName.name,new table_t(vfTName.name));
    	    else
    	        vpTableList.put(vfTName.cname,new table_t(vfTName.cname));
    	}

        //proses field
        if(!vpPO.selectWildcard)
        {
        	for (Enumeration vfEn = vpPO.field.elements() ; vfEn.hasMoreElements() ;)
        	{
        	    field_t vfField=(field_t)vfEn.nextElement();
            	table_t vfTable=(table_t)vpTableList.get(vfField.table);
        	    field_t vfFieldx=new field_t(vfField.table,vfField.name,true);
            	vfTable.field.put(vfField.name,vfFieldx);
            	vfTable.fieldorder.addElement(vfFieldx);
            	vfTable.project.addElement(vfFieldx);
            }
        }

        //proses order
    	for (Enumeration vfEn = vpPO.order.elements() ; vfEn.hasMoreElements() ;)
    	{
    	    order_t vfOrder=(order_t)vfEn.nextElement();
        	table_t vfTable=(table_t)vpTableList.get(vfOrder.table);
        	field_t vfField;
        	if((vfField=(field_t)vfTable.field.get(vfOrder.name))==null)
        	{
        		vcError.fPutError(1802,"OptimizerScheduler Error : ORDER BY identifier must exist in SELECT argumen: "+vfOrder.table+"."+vfOrder.name,true);
//        		Trace(TRACE_OUT,"msqlProcessQuery()");
        		return;
        	}
        	vfField.order=true;
        	vfTable.order.addElement(vfField);
        }
        //proses cond
    	for (Enumeration vfEn = vpPO.cond.elements() ; vfEn.hasMoreElements() ;)
    	{
    	    cond_t vfCond=(cond_t)vfEn.nextElement();
    	    field_t vfField;
        	table_t vfTable=(table_t)vpTableList.get(vfCond.table);
        	if((vfField=(field_t)vfTable.field.get(vfCond.name))==null)
        	{
        	    vfField=new field_t(vfCond.table,vfCond.name);
        	    vfTable.field.put(vfCond.name,vfField);
        	    vfTable.fieldorder.addElement(vfField);
        	}

            if(vfCond.value.type==IDDBConstants.IDENT_TYPE)
            {
        	    vfField.condJoin=true;
                vfTable.condJoin.addElement(vfCond);
        	    vfField.cond=vfCond;

            	vfTable=(table_t)vpTableList.get(vfCond.value.identVal.seg1);
            	if((vfField=(field_t)vfTable.field.get(vfCond.value.identVal.seg2))==null)
            	{
            	    vfField=new field_t(vfCond.value.identVal.seg1,vfCond.value.identVal.seg2);
            	    vfTable.field.put(vfCond.value.identVal.seg2,vfField);
            	    vfTable.fieldorder.addElement(vfField);
            	}
                vfTable.condJoin.addElement(vfCond);
        	    vfField.condJoin=true;
        	    vfField.cond=vfCond;
            }
            else
            {
        	    vfField.condSelect=true;
                vfTable.condSelect.addElement(vfCond);
            }
        }

    	for (Enumeration vfEn = vpTableList.elements() ; vfEn.hasMoreElements() ;)
    	{
    	    table_t vfTable=(table_t)vfEn.nextElement();
    	    if (vfTable==null)
    	        vpTableListSort.addElement(vfTable);
    	    else
    	    {
    	        int vfI;
        	    for(vfI=0;vfI<vpTableListSort.size();vfI++)
        	    if(((table_t)vpTableListSort.elementAt(vfI)).condJoin.size()>vfTable.condJoin.size())
        	        break;
        	    vpTableListSort.insertElementAt(vfTable,vfI);
    	    }
    	}
    /*    for(int i=0;i<max-1;i++)
        {
            for(int j=i+1;j<max;j++)
            {
                table_t ti=(table_t) vpTableList.elementAt(i);
                table_t tj=(table_t) vpTableList.elementAt(j);
                if(ti.condJoin.size()>
                   tj.condJoin.size())
                {
                    vpTableList.setElementAt(tj,i);
                    vpTableList.setElementAt(ti,j);
                }
            }
        }

    */
        if(vcDebugLevel>=1)
            System.out.println("OUT processtablelist");
    }
}