/*
**	SqlProcedure.java
**
**
** Copyright (c) 1993-95  David J. Hughes
** Copyright (c) 1995  Hughes Technologies Pty Ltd
**
** Permission to use, copy, and distribute for non-commercial purposes,
** is hereby granted without fee, providing that the above copyright
** notice appear in all copies and that both the copyright notice and this
** permission notice appear in supporting documentation.
**
** This software is provided "as is" without any expressed or implied warranty.
**
** ID = "$Id:"
**
** modified by Junus Junarto D (junus@its.ac.id)
** for IDDB (ITS Distributed DataBase)
** Computer Science Department
** Institut Teknologi Sepuluh Nopember
**
*/

/***************************************************************************
** Internal structures
*/


/*
** Where clause list element
*/

import java.io.*;
import java.util.*;
import java.sql.*;

class ParserProcedure
{
    public parserout_t po;
    private int fieldLast=0;
    private boolean havePriKey=false;
    private int vcDebugLevel=0;
    private ErrorManager vcError;

    public ParserProcedure(ErrorManager vpError, int vpDebugLevel)
    {
        vcError=vpError;
        vcDebugLevel=vpDebugLevel;
    }

    public void reinit(parserout_t po)
    {
        this.po = po;
        fieldLast=0;
        havePriKey = false;
    }

/****************************************************************************
** 	_Clean - clean out the internal structures
**
**	Purpose	: Free all space and reset structures after a query
**	Args	: None
**	Returns	: Nothing
**	Notes	: Updates all public data structures
*/


void Clean()
{

    Trace(IDDBConstants.TRACE_IN,"Clean()");

    po = null;
    fieldLast=0;
    havePriKey = false;
/*    command = COMMAND_EMPTY;
    havePriKey = false;
    SelectLimit=0;

    selectDistinct=false;
	selectWildcard=false;
	fieldName=false;
    fieldLast=0;

    table.clear();
    field.removeAllElements();
    cond.removeAllElements();
    order.clear();

	Trace(TRACE_OUT,"Clean()");
*/
}


ident_t CreateIdent(String seg1, String seg2)
{
	ident_t	i_new;

	Trace(IDDBConstants.TRACE_IN,"CreateIdent()");
    if (seg1!=null)
	{
		if (seg1.length() > IDDBConstants.NAME_LEN)
		{
			ErrorWrite("Identifier name too long",seg1);
			Trace(IDDBConstants.TRACE_OUT,"CreateIdent()");
			return(null);
		}
	}
	if (seg2!=null)
	{
		if (seg2.length() > IDDBConstants.NAME_LEN)
		{
			ErrorWrite("Identifier name too long",seg2);
			Trace(IDDBConstants.TRACE_OUT,"CreateIdent()");
			return(null);
		}
	}
	i_new = new ident_t(seg1,seg2);
	Trace(IDDBConstants.TRACE_OUT,"CreateIdent()");
	return i_new;
}


val_t CreateValue(Object textRep,int type, int tokLen)
{
	val_t	v_new;

	Trace(IDDBConstants.TRACE_IN,"CreateValue()");
	v_new = new val_t();
	v_new.type = type;
	v_new.dataLen = tokLen;
	switch(type)
	{
		case IDDBConstants.NULL_TYPE:
			v_new.nullVal = true;
			break;
		case IDDBConstants.IDENT_TYPE:
			v_new.identVal = (ident_t)textRep;
			break;

		case IDDBConstants.CHAR_TYPE:
			v_new.charVal = (String) textRep;
			break;

		case IDDBConstants.INT_TYPE:
			v_new.intVal = Integer.valueOf((String) textRep).intValue();
			break;

		case IDDBConstants.FLOAT_TYPE:
			v_new.floatVal = Float.valueOf((String) textRep).floatValue();
			break;
	}
	Trace(IDDBConstants.TRACE_OUT,"CreateValue()");
	return v_new;
}


val_t fillValue(String val, int type, int length)
{
	val_t	v_new;

	Trace(IDDBConstants.TRACE_IN,"fillValue()");
	v_new = new val_t ();
	v_new.type = type;
	v_new.nullVal = false;
	switch(type)
	{
		case IDDBConstants.CHAR_TYPE:
			v_new.charVal= val;
			break;

		case IDDBConstants.INT_TYPE:
			v_new.intVal = Integer.valueOf((String) val).intValue();
			break;

		case IDDBConstants.FLOAT_TYPE:
			v_new.floatVal = Float.valueOf((String) val).floatValue();
			break;
	}
    Trace(IDDBConstants.TRACE_OUT,"fillValue()");
	return(v_new);
}


val_t nullValue()
{
	val_t	v_new;
	v_new = new val_t();
	v_new.nullVal = true;
	return(v_new);
}

void Trace(int i, String s)
{
    if(vcDebugLevel<2) return;
    switch(i)
    {
        case IDDBConstants.TRACE_IN :
                    System.out.println("Trace in  : "+s);
                    break;
        case IDDBConstants.TRACE_OUT :
                    System.out.println("Trace out : "+s);
                    break;
    }
}

void ErrorWrite(String s1, String s2)
{
    vcError.fPutError(1001,s1+s2,true);
}



/****************************************************************************
** 	_msqlAddField - add a field definition to the list
**
**	Purpose	: store field details from the query for later use
**	Args	: field name, field type, field length, value
**	Returns	: Nothing
**	Notes	: Depending on the query in process, only some of the
**		  args will be supplied.  eg. a SELECT doesn't use the
**		  type arg.  The length arg is only used during a create
**		  if the field is of type CHAR
*/

int AddField(ident_t ident,int type,int length,boolean notNull,boolean priKey)
{
	field_t	f_new;
	String name, table;

	Trace(IDDBConstants.TRACE_IN,"AddField()");

	name = ident.seg2;
	table = ident.seg1;

	if (priKey)
	{
		if (havePriKey)
		{
			ErrorWrite("Can't have multiple primary keys","");
			Trace(IDDBConstants.TRACE_OUT,"AddField()");
			return(-1);
		}
		else
		{
			havePriKey = true;
		}
	}


	f_new = new field_t();
	if (table!=null)
	{
		if (table.length() > IDDBConstants.NAME_LEN)
		{
			ErrorWrite("Table name is too long", table);
			Trace(IDDBConstants.TRACE_OUT,"AddField()");
			return(-1);
		}
		f_new.table=table;
	}
	if (name.length() > IDDBConstants.NAME_LEN)
	{
		ErrorWrite("Field name is too long", name);
		Trace(IDDBConstants.TRACE_OUT,"AddField()");
		return(-1);
	}
	f_new.name=name;
	f_new.primarykeyflag = priKey;
	f_new.notnullflag = notNull;

	switch(type)
	{
		case IDDBConstants.INT_TYPE:
			f_new.type = Types.INTEGER;
			f_new.length = 4;
			break;

		case IDDBConstants.CHAR_TYPE:
			f_new.type = Types.CHAR;
			f_new.length = length;
			break;

		case IDDBConstants.FLOAT_TYPE:
			f_new.type = Types.FLOAT;
			f_new.length = 8;
			break;
		default:
			f_new.type = 0;
			f_new.length = 0;
			break;
	}
	boolean ffound=false;
	for(int i=0;(i<po.field.size())&&!ffound;i++)
	{
	    field_t f=(field_t)po.field.elementAt(i);
	    if (f.name.equals(name)&&f.table.equals(table))
    	{
    	    ffound=true;
    		ErrorWrite("Duplicate po.field name ",name);
    		Trace(IDDBConstants.TRACE_OUT,"AddField()");
    		return(-1);
    	}
	}
	if(!ffound)
	{
	  po.field.addElement(f_new);
	  fieldLast++;
	}
	Trace(IDDBConstants.TRACE_OUT,"AddField()");
	return(0);
}

void SetSelectLimit(val_t value)
{
	po.SelectLimit = value.intVal;
}


void AddFieldValue(val_t value)
{
	Trace(IDDBConstants.TRACE_IN,"AddFieldValue()");
	field_t	fieldVal;
	if(po.fieldName)
	{
	    fieldVal=new field_t();
	    po.field.addElement(fieldVal);
	}
	else
	if(po.field.size()>=fieldLast)
	    fieldVal=(field_t)po.field.elementAt(fieldLast-1);
	else
	{
		ErrorWrite("Number name < number po.field","");
		Trace(IDDBConstants.TRACE_OUT,"AddFieldValue()");
		return;
	}


	if (fieldVal!=null)
	{
		fieldVal.value = value;
	}
	Trace(IDDBConstants.TRACE_OUT,"AddFieldValue()");
}




/****************************************************************************
** 	_msqlAddCond  -  add a conditional spec to the list
**
**	Purpose	: Store part of a "where_clause" for later use
**	Args	: po.field name, test op, value, bool (ie. AND | OR)
**	Returns	: Nothing
**	Notes	: the BOOL po.field is only provided if this is not the first
**		  element of a where_clause.
*/

int AddCond(ident_t ident,int op,val_t value,int bool)
{
    cond_t	c_new;
	String name,
		table;

	Trace(IDDBConstants.TRACE_IN,"AddCond()");

	name=ident.seg2;
	table = ident.seg1;

	if (name.length() > IDDBConstants.NAME_LEN)
	{
		ErrorWrite("po.field name is too long", name);
		Trace(IDDBConstants.TRACE_OUT,"AddCond()");
		return(-1);
	}
	if (table!=null)
	{
		if (table.length() > IDDBConstants.NAME_LEN)
		{
			ErrorWrite("Table name is too long", name);
			Trace(IDDBConstants.TRACE_OUT,"msqlAddCond()");
			return(-1);
		}
	}
	c_new=new cond_t();
	c_new.name = name;
	c_new.table = table;
	c_new.op = op;
	c_new.bool = bool;
	c_new.value = value;

    po.cond.addElement(c_new);
	Trace(IDDBConstants.TRACE_OUT,"AddCond()");
	return(0);

}



/****************************************************************************
** 	_msqlAddOrder  -  add an order definition to the list
**
**	Purpose	: Store part of an "order_clause"
**	Args	: po.field name, order direction (ie. ASC or DESC)
**	Returns	: Nothing
**	Notes	:
*/

void AddOrder(ident_t ident,int dir)
{
	order_t	o_new;

	Trace(IDDBConstants.TRACE_IN,"AddOrder()");
    o_new = new order_t();
	o_new.table=ident.seg1;
	o_new.name=ident.seg2;
	o_new.dir = dir;
	po.order.put(ident.seg1+"."+ident.seg2,o_new);
	Trace(IDDBConstants.TRACE_OUT,"AddOrder()");
}




int AddTable(String name,String alias)
{
    tname_t	t_new, cur;

	Trace(IDDBConstants.TRACE_IN,"AddTable()");

	t_new = new tname_t();
	if (alias!=null)
	{
	    if(po.table.containsKey(alias))
			{
				ErrorWrite("Nonunique po.table/alias name ",alias);
				Trace(IDDBConstants.TRACE_OUT,"AddTable()");
				return(-1);
			}

		t_new.name=alias;
		t_new.cname=name;
	if (alias.length() > IDDBConstants.NAME_LEN)
	{
		ErrorWrite("po.table alias name is too long",name);
		Trace(IDDBConstants.TRACE_OUT,"AddTable()");
		return(-1);
	}
	}
	else
	{
	    if(po.table.containsKey(name))
		{
			ErrorWrite("Nonunique po.table name: ",name);
			Trace(IDDBConstants.TRACE_OUT,"AddTable()");
			return(-1);
		}
		t_new.name=name;
		t_new.cname = null;
	}

	if (name.length() > IDDBConstants.NAME_LEN)
	{
		ErrorWrite("po.table name is too long",name);
		Trace(IDDBConstants.TRACE_OUT,"AddTable()");
		return(-1);
	}
	po.table.put(t_new.name,t_new);
	Trace(IDDBConstants.TRACE_OUT,"AddTable()");

	return(0);
}






void SetDB(String db)
{
	po.curDB = db;
}

void processAlias()
{
    boolean HarusProsesNamaTable=(po.table.size()>1 ? true:false);

    //proses po.field
    if(!po.selectWildcard)
    {
    	for (Enumeration e = po.field.elements() ; e.hasMoreElements() ;)
    	{
    	    field_t f=(field_t)e.nextElement();
    	    if((f.table==null)&&HarusProsesNamaTable)
        	{
        		ErrorWrite("Number po.table more than one, must use name/alias to po.field identifier","");
        		Trace(IDDBConstants.TRACE_OUT,"msqlProcessQuery()");
        		return;
        	}
    	    else
    	    {
    	        if(f.table!=null)
    	        {
            	    tname_t t=(tname_t)po.table.get(f.table);
            	    if(t==null)
                	{
                		ErrorWrite("No po.table name/alias match from FROM clause",f.table);
                		Trace(IDDBConstants.TRACE_OUT,"msqlProcessQuery()");
                		return;
                	}
                	else
                	{
                	    if(t.cname!=null)
                	        f.table=t.cname;
                	}
            	}
            }
        }
    }

    //proses po.order
	for (Enumeration e = po.order.elements() ; e.hasMoreElements() ;)
	{
	    order_t o=(order_t)e.nextElement();
	    if((o.table==null)&&HarusProsesNamaTable)
    	{
    		ErrorWrite("Number po.table more than one, must use name/alias to po.field identifier","");
    		Trace(IDDBConstants.TRACE_OUT,"msqlProcessQuery()");
    		return;
    	}
	    else
	    {
    	    tname_t t=(tname_t)po.table.get(o.table);
    	    if(t==null)
        	{
        		ErrorWrite("No po.table name/alias match from po.order BY clause",o.table);
        		Trace(IDDBConstants.TRACE_OUT,"msqlProcessQuery()");
        		return;
        	}
        	else
        	{
        	    if(t.cname!=null)
        	        o.table=t.cname;
        	}
        }
    }
    //proses po.cond
	for (Enumeration e = po.cond.elements() ; e.hasMoreElements() ;)
	{
	    cond_t c=(cond_t)e.nextElement();
	    if((c.table==null)&&HarusProsesNamaTable)
    	{
    		ErrorWrite("Number po.table more than one, must use name/alias to po.field identifier","");
    		Trace(IDDBConstants.TRACE_OUT,"msqlProcessQuery()");
    		return;
    	}
	    else
	    {
    	    tname_t t=(tname_t)po.table.get(c.table);
    	    if(t==null)
        	{
        		ErrorWrite("No po.table name/alias match from WHERE clause",c.table);
        		Trace(IDDBConstants.TRACE_OUT,"msqlProcessQuery()");
        		return;
        	}
        	else
        	{
        	    if(t.cname!=null)
        	        c.table=t.cname;
            	if(c.value.type==IDDBConstants.IDENT_TYPE)
            	{
            	    if((c.value.identVal.seg1==null)&&HarusProsesNamaTable)
                	{
                		ErrorWrite("Number po.table more than one, must use name/alias to po.field identifier","");
                		Trace(IDDBConstants.TRACE_OUT,"msqlProcessQuery()");
                		return;
                	}
            	    else
            	    {
                	    tname_t x=(tname_t)po.table.get(c.value.identVal.seg1);
                	    if(x==null)
                    	{
                    		ErrorWrite("No po.table name/alias match from WHERE clause",c.value.identVal.seg1);
                    		Trace(IDDBConstants.TRACE_OUT,"msqlProcessQuery()");
                    		return;
                    	}
                    	else
                    	{
                    	    if(x.cname!=null)
                    	        c.value.identVal.seg1=x.cname;

                    	}
                    }
                }
            }
        }
    }
}

/****************************************************************************
** 	_msqlProcessQuery  -  send to query to the approp routine
**
**	Purpose	: Call the required routine to build the native query
**	Args	: None
**	Returns	: Nothing
**	Notes	: Global command variable used.  This is called when the
**		  end of an individual query is found in the miniSQL code
**		  and provides the hooks into the per-database routines.
*/



void ProcessQuery()
{
	int	res;

	Trace(IDDBConstants.TRACE_IN,"ProcessQuery()");
/*	if (curDB!=null)
	{
		ErrorWrite("No current database");
		Debug(MOD_ERR,"No current database");
		Trace(IDDBConstants.TRACE_OUT,"ProcessQuery()");
		return;
	}
*/

    if(vcDebugLevel>=2)
    {
        if(po.command==IDDBConstants.COMMAND_SELECT)
            processAlias();

    	System.out.println("\n\nRESULT\n\nCOMMAND : "+po.command);
    	System.out.println("\nTABLE :");
    	for (Enumeration e = po.table.elements() ; e.hasMoreElements() ;)
    	{
    	    tname_t t=(tname_t)e.nextElement();
        	System.out.print("Nama :"+t.name+" |");
        	System.out.println("CNama :"+t.cname);
        }

    	System.out.println("\nCONDITION :");
    	for (Enumeration e = po.cond.elements() ; e.hasMoreElements() ;)
    	{
    	    cond_t t=(cond_t)e.nextElement();
        	System.out.print("po.table :"+t.table+" |");
        	System.out.print("Nama :"+t.name+" |");
        	System.out.print("Op :"+t.op+" |");
        	System.out.print("Bool :"+t.bool+" |");
        	System.out.print("Type :"+t.type+" |");
        	System.out.println("Length :"+t.length);
        	if(t.value!=null)
        	{
        	    System.out.print("   value (intVal):"+t.value.intVal+" |");
        	    System.out.print("(charVal):"+t.value.charVal+" |");
        	    System.out.print("(floatVal):"+t.value.floatVal+" |");
        	    System.out.print("(type):"+t.value.type+" |");
        	    System.out.print("(dataLen):"+t.value.dataLen+" |");
        	    System.out.println("\n         (nullVal):"+t.value.nullVal);
        	    if(t.value.identVal!=null)
        	    {
            		System.out.print("         identVal (seg1):"+t.value.identVal.seg1+" |");
            		System.out.println("(seg2):"+t.value.identVal.seg2);
        	    }
        	}
        }

    	System.out.println("\nORDER :");
    	for (Enumeration e = po.order.elements() ; e.hasMoreElements() ;)
    	{
    	    order_t t=(order_t)e.nextElement();
    	System.out.print("po.table :"+t.table+" |");
    	System.out.print("Nama :"+t.name+" |");
    	System.out.print("Dir :"+t.dir+" |");
    	System.out.print("Type :"+t.type+" |");
    	System.out.println("Length :"+t.length);
        }

    	System.out.println("\nFIELD :");
    	for (Enumeration e = po.field.elements() ; e.hasMoreElements() ;)
    	{
    	    field_t t=(field_t)e.nextElement();
    	System.out.print("po.table :"+t.table+" |");
    	System.out.print("Nama :"+t.name+" |");
    	System.out.print("Type :"+t.type+" |");
    	System.out.print("Length :"+t.length+" |");
    	System.out.print("Notnullflag :"+t.notnullflag+" |");
    	System.out.println("Primarykeyflag :"+t.primarykeyflag);
    	if(t.value!=null)
    	{
    	    System.out.print("   value (intVal):"+t.value.intVal+" |");
    	    System.out.print("(charVal):"+t.value.charVal+" |");
    	    System.out.print("(floatVal):"+t.value.floatVal+" |");
    	    System.out.print("(type):"+t.value.type+" |");
    	    System.out.print("(dataLen):"+t.value.dataLen+" |");
    	    System.out.println("\n         (nullVal):"+t.value.nullVal);
    	    if(t.value.identVal!=null)
    	    {
    		System.out.print("         identVal (seg1):"+t.value.identVal.seg1+" |");
    		System.out.println("(seg2):"+t.value.identVal.seg2);
    	    }
    	}
    }
}
}

}
