import java.sql.*;
import java.util.*;

class Replicator implements Runnable
{
/*

    synchronous :
        setiap kali saat perintah select dilakukan operasi compare
    asynchronous :
        setiap kelipatan waktu tertentu baru dilakukan operasi compare

    operasi compare : item-item dalam table log di RA dan repl-pusat dibandingkan.
        ambil semua daftar tabel
            ambil semua site-site tabel tersebut
                ambil jml item di log RA
                ambil jml item di log repl-pusat
                jika jml item log RA > repl-pusat ambil item-item di log RA dengan nomor > jml item log repl-pusat
                masukkan item-item tersebut ke tabel yang di tuju. *
                update jml item log repl-pusat.

    IDDBTBCOUNT
        di RA berisi jumlah log di RA
        di pusat berisi jumlah log global (*) dan tiap-tiap RA

*/
	private ErrorManager vcError;
	private int vcDebugLevel;

    private Thread vcReplThread;
    private long vcSleep;
    private Executor vcExecutor;
    private Connection vcConnection;
	public boolean vcLock;
	public boolean vcLockReplicator;


    Replicator(long vpSleep, Executor vpExecutor, Connection vpConnection, ErrorManager vpError, int vpDebugLevel)
    {
        vcSleep=vpSleep;
        vcExecutor=vpExecutor;
        vcError=vpError;
        vcDebugLevel=vpDebugLevel;
        vcConnection=vpConnection;
        vcLock=true;
        vcLockReplicator=false;
        vcReplThread = new Thread(this);
    }

    public void run()
    {
        while(true)
        {
            try
            {
                vcReplThread.sleep(vcSleep);
    		} catch (InterruptedException vfEx) {
    		    vcError.fPutError(1601,"Replicator Error : Running sleep Thread ! Exception: " + vfEx.toString());
    	    }

            while(vcLock);

            if(vcDebugLevel>=1)
                System.out.println("***********IN****** REPLICATOR *********************");
            fReplication();
            if(vcDebugLevel>=1)
                System.out.println("***********OUT***** REPLICATOR *********************");
        }
    }

    public void start() { vcReplThread.start(); }

    public void resume() { vcReplThread.resume(); }

    public void stop() { vcReplThread.stop(); }

    public void close()
    {
        while(vcLockReplicator);
        stop();
    }

    public void fReplication()
    {
        vcLockReplicator=true;
        try
        {
      		Statement vfSUpdate = vcConnection.createStatement();
      		Statement vfSQuery = vcConnection.createStatement();
            ResultSet vfRS = vfSQuery.executeQuery("select itbname, isiteid from iddbtbsite");
            String vfTName ;
            String vfSiteId ;
            while(vfRS.next())
            {
                vfTName=vfRS.getString(1).trim();
                vfSiteId=vfRS.getString(2).trim();

                ResultSet vfRS2 = vfSUpdate.executeQuery("select icounter from iddbtbcount"+
                    " where isiteid = '"+vfSiteId+"' and itbname = '"+vfTName+"'");
                vfRS2.next();
                int vfTbCount=vfRS2.getInt(1);
                vcExecutor.fPutRASingle("select icounter from iddbtbcount"+
                    " where isiteid = '"+vfSiteId+"' and itbname = '"+vfTName+"'",IDDBConstants.COMMAND_SELECT,vfSiteId);
                // DBMS lokal belum pernah ada isinya.
                if(vcExecutor.vcRSOut.RSi==-2)
                {
                    vcExecutor.fPutRASingle("create table iddbtbcount (itbname char(20) not null,"+
                        "isiteid char(20) not null, icounter integer not null)",IDDBConstants.COMMAND_CREATE_TABLE,vfSiteId);
                    if(vcExecutor.vcRSOut.RSi==-2)
                        vcError.fPutError(1604,"Replicator Error : Tabel: iddbtbcount already created at site: "+vfSiteId);
                    else
                        vcError.fPutError(1605,"Replicator Error : Create Tabel: iddbtbcount at site: "+vfSiteId);
                    vcExecutor.fPutRASingle("insert into iddbtbcount values('"+vfTName+"', '"+vfSiteId+"',"+vfTbCount+")"
                        ,IDDBConstants.COMMAND_INSERT,vfSiteId);

                    //buat table di remote agent
                    String vfQuery="create table "+vfTName+" (";
                    vfRS=vfSQuery.executeQuery("select * from iddbtbfield where itbname = '"+vfTName+"' order by ino");
                    String vfQueryTmp="";
                    Vector vfFieldType=new Vector();
                    while(vfRS.next())
                    {
                        if(!vfQueryTmp.equals(""))
                            vfQueryTmp+=", ";
                        vfQueryTmp+=vfRS.getString(2)+ " ";
                        int vfFType=vfRS.getInt(3);
                        switch (vfFType)
                        {
                            case Types.CHAR:
                                vfQueryTmp+="char("+vfRS.getInt(4)+")";
                                break;
                            case Types.INTEGER:
                                vfQueryTmp+="integer";
                                break;
                            case Types.FLOAT:
                                vfQueryTmp+="float";
                                break;
                        }
                        vfFieldType.addElement(new Integer(vfFType));
                    }
                    vfQuery+=vfQueryTmp+")";

                    vcExecutor.fPutRASingle(vfQuery,IDDBConstants.COMMAND_CREATE_TABLE,vfSiteId);
                    if(vcExecutor.vcRSOut.RSi==-2)
                        vcError.fPutError(1604,"Replicator Error : Tabel: "+vfTName+" already created at site: "+vfSiteId);
                    else
                        vcError.fPutError(1605,"Replicator Error : Create Tabel: "+vfTName+" at site: "+vfSiteId);

                    vfRS=vfSQuery.executeQuery("select * from "+vfTName+" where isiteid='"+vfSiteId+"'");
                    vfQuery="";

                    while(vfRS.next())
                    {
                        int vfI=1;
                        for(Enumeration vfEn = vfFieldType.elements();vfEn.hasMoreElements() ;)
                        {
                            int vfFType = ((Integer)vfEn.nextElement()).intValue();
                            if(!vfQuery.equals(""))
                                vfQuery+=", ";
                            switch (vfFType)
                            {
                                case Types.CHAR:
                                    vfQuery+="'"+vfRS.getString(vfI)+"'";
                                    break;
                                case Types.INTEGER:
                                    vfQuery+=vfRS.getInt(vfI);
                                    break;
                                case Types.FLOAT:
                                    vfQuery+=vfRS.getFloat(vfI);
                                    break;
                            }
                            vfI++;

                        }
                        vfQuery="insert into "+vfTName+" values ("+vfQuery+")";
                        if(vcDebugLevel>=1)
                            System.out.println(vfQuery);
                        vfSUpdate.executeUpdate(vfQuery);
                    }
                }
                else
                {

                    ResultSet vfRSRA = (ResultSet) vcExecutor.vcRSOut.RS.elementAt(0);
                    vfRSRA.next();
                    int vfTbCountRA=vfRSRA.getInt(1);

                    if(vfTbCount<vfTbCountRA)
                    {
                        if(vcDebugLevel>=1)
                            System.out.println("***REPLICATOR*** UPDATE different at isiteid : "+vfSiteId);

                        vcExecutor.fPutRASingle("select * from log"+vfTName+" where icounter>="+vfTbCount+" and isiteid<>'*' order by icounter",IDDBConstants.COMMAND_SELECT,vfSiteId);
                        vfRSRA =(ResultSet) vcExecutor.vcRSOut.RS.elementAt(0);

                        while(vfRSRA.next())
                        {
                            int vfTbCountX=vfRSRA.getInt(2);
                            if(vfTbCount!=vfTbCountX)
                            {
                                vcError.fPutError(1602,"Replicator Error : Different operation comparation log number ! : TbCount: "+vfTbCount+" TbCountX: "+vfTbCountX);
                                return;
                            }
                            String vfIOperationRA=vfRSRA.getString(3).trim();
                            int vfIType=vfRSRA.getInt(4);
                            String vfQueryUpdate=null;
                            if(vfIType==IDDBConstants.COMMAND_INSERT)
                            {
                                vfQueryUpdate=vfIOperationRA.substring(0,vfIOperationRA.lastIndexOf(")"))+", '"+vfSiteId+"')";
                            }
                            else
                            {
                                if(vfIOperationRA.indexOf("where")==-1)
                                    vfQueryUpdate=vfIOperationRA + " where isiteid = '"+vfSiteId+"'";
                                else
                                    vfQueryUpdate=vfIOperationRA + " and isiteid = '"+vfSiteId+"'";
                            }
                            if(vcDebugLevel>=2)
                                System.out.println("***REPLICATOR*** Query : "+vfQueryUpdate);
                            vfSUpdate.executeUpdate(vfQueryUpdate);

                            vfSUpdate.executeUpdate("insert into log"+vfTName+" values('"+
                                (new java.util.Date(System.currentTimeMillis())).toString()+
                                "',"+ vfTbCount++ +",'"+Tools.fReplaceString(vfIOperationRA,"'","''")+"',"+
                                vfIType+",'"+vfRSRA.getString(5).trim()+"','"+vfSiteId+"')");
                        }
                        vfSUpdate.executeUpdate("update iddbtbcount set icounter='"+vfTbCount+"'"+
                        " where isiteid = '"+vfSiteId+"' and itbname = '"+vfTName+"'");
                    }
                }

            }
            vfSUpdate.close();
            vfSQuery.close();
		} catch (SQLException vfEx) {
		    vcError.fPutError(1603,"Replicator Error in SQL statement, Exception: " + vfEx.toString());
	    }
        vcLockReplicator=false;
    }
}