import java.util.*;
import java.io.*;
import java.sql.*;

class command_t
{
    public trans_t trans;
    public Integer communicatorNo;
    public String command;
    public parserout_t po;
    public int counter;

    public command_t(Integer communicatorNo, String command, parserout_t po)
    {
        this.communicatorNo=communicatorNo;
        this.command=command;
        this.po=po;
    }
}


class trans_t
{
    public Integer transNo;
    public Integer communicatorNo;
    public Vector command=new Vector();
    public boolean commit=true;

    public trans_t(Integer transNo,Integer communicatorNo)
    {
        this.transNo=transNo;
        this.communicatorNo=communicatorNo;
    }
}

class variable_t
{
    public String str;
    int tsRead=-1;
    int tsUpdate=-1;
    int tsUpdateCommitQCounter=-1;

//    public SortVector read=new SortVector();
    public SortVector update=new SortVector();

    public variable_t(String str)
    {
        this.str=str;
    }
}


class SortVector extends Vector
{
    public synchronized void putElement(command_t obj)
    {
        int i=0;
        boolean insert=false;
        for (Enumeration e = elements() ; e.hasMoreElements() ;i++)
        {
            command_t o = (command_t) e.nextElement();
            if(o.trans.transNo.intValue()>obj.trans.transNo.intValue())
            {
                insert=true;
                insertElementAt(obj,i);
                break;
            } else if(o.trans.transNo.intValue()==obj.trans.transNo.intValue())
                break;
        }
        if(!insert)
          addElement(obj);
    }
}

class TransactionManager
{
    private ErrorManager vcError;
	private int vcDebugLevel;

    private Hashtable vcQueueTrans= new Hashtable();
    private Hashtable vcQueueVariable= new Hashtable();

    private int vcTransNoCounter=0;
    private int vcQueryNo=-1;

    private int vcDelay=1000;

    private Parser vcParser;
    private OptimizerScheduler OptSch;
    private Statement vcStatement;
    private Executor vcExecutor;

	private int vcTransType=IDDBConstants.TRANS_CONCURRENT;

    private int vcLogCounter;

    private UserAuthentication vcUserAuthentication;

    public TransactionManager(Statement vpStatement, UserAuthentication vpUserAuthentication, int vpTransType, int vpOptimizerType,
        ErrorManager vpError, int vpDebugLevel)
    {
        vcUserAuthentication=vpUserAuthentication;

        vcError=vpError;
        vcDebugLevel=vpDebugLevel;
        vcTransType=vpTransType;

        vcStatement = vpStatement;

        try
        {
      	    ResultSet vfRS=vcStatement.executeQuery("select icounter from iddbtbcount where itbname='log' and isiteid='*'");
      	    if(vfRS.next())
      	        vcLogCounter=vfRS.getInt(1);
      	    else
      	        vcLogCounter=0;
		} catch(SQLException vfEx)
		{
	        vcError.fPutError(1301,"Transaction Manager Error : Initialisation Fail, Exception: " + vfEx.toString());
		}


        OptSch = new OptimizerScheduler(vpStatement,vpOptimizerType,vpError,vpDebugLevel);
    }

    public void fInsertExecutor(Executor vpExecutor)
    {
        vcExecutor=vpExecutor;
        OptSch.fInsertExecutor(vpExecutor);
    }

    private parserout_t getParser(String vpQuery)
    {
        parserout_t vfPO=null;
        if(vcParser==null)
        {
            vcParser = new Parser(new StringBufferInputStream(vpQuery+";"));
            vcParser.insertError_DebugLevel(vcError,vcDebugLevel);
            if(vcDebugLevel<2)
                vcParser.disable_tracing();
        }
        else
        {
            vcParser.ReInit(new StringBufferInputStream(vpQuery+";"));
        }

        try
        {
            vfPO=vcParser.query();
            if(vcDebugLevel>=1)
                System.out.println("Sql grammar parsed successfully.");
        } catch (ParseException vfEx) {
	        vcError.fPutError(1101,"Parser Error : Encounter Error during parsing, Exception: " + vfEx.toString(),true);
        }
        return vfPO;
    }

    public ResultOut fRemoveTransaction(Integer vpConnectorNo,boolean vpAutoCommit,int vpTransactionLevel,String vpCommit) //throws SQLException
    {
        if(vpCommit!=null)
            vcQueryNo++;
        ResultOut vfRSOut=null;
        int vfValueReturn=0;
        // cek di queue update apakah masih ada update sebelumnya
        // yang belum diupdate, kalau ada tunggu sampai waktu tertentu, kalau masih
        // belum selesai keluar, COMMIT=-1, atau COMMIT=0;
        if(!vpAutoCommit&&(vcTransType!=IDDBConstants.TRANS_NONE))
        {
            // cek di queue update apakah masih ada update sebelumnya
            // yang belum diupdate, kalau ada tunggu sampai waktu tertentu, kalau masih
            // belum selesai keluar, COMMIT=-1, atau COMMIT=0;
            trans_t vfTrans=(trans_t) vcQueueTrans.get(vpConnectorNo);
            int vfTransNo=vfTrans.transNo.intValue();

            for (Enumeration vfEn = vfTrans.command.elements() ; vfEn.hasMoreElements() ;)
            {
                command_t vfCommand= (command_t) vfEn.nextElement();

                // BEBERAPA VARIABLE
                for(Enumeration vfEn2 = vfCommand.po.table.elements() ; vfEn2.hasMoreElements() ;)
                {
                    String vfTable;
                    tname_t vfTName= (tname_t) vfEn2.nextElement();
                    if(vfTName.cname!=null)
            	        vfTable=vfTName.cname;
            	    else
            	        vfTable=vfTName.name;

                    variable_t vfVariable = (variable_t)vcQueueVariable.get(vfTable);


                    if(vfTrans.commit)
                    {
                        long vfStartTime=System.currentTimeMillis();
                        long vfCounter=0;
                        if(vfCommand.po.command==IDDBConstants.COMMAND_SELECT)
                        {
                            if(vfVariable.update.size()>0)
                            {
                                while((((command_t)vfVariable.update.firstElement()).trans.transNo.intValue()
                                      <vfTransNo)&&(vfCounter<vcDelay))
                                    vfCounter=System.currentTimeMillis()-vfStartTime;
                            }
                            if(vfCounter>=vcDelay)
                            {
                                vfValueReturn=-1;
                                vfTrans.commit=false;
                                vcError.fPutError(1302,"Transaction Manager Error : BLOCKED by query 'update: "+vfVariable.str+"' from Transaction No: "+((command_t)vfVariable.update.firstElement()).trans.transNo.intValue()+
                                    " "+vpCommit+" '"+vfCommand.command+"' ROLLBACK READ this transaction: "+vfTransNo+", because will not be commited",true);
                            }
                            // cek apakah ada update dari trans sesudahnya, kalau ada rollback
                            // ts(Ti)<update(x)
                            else if(vfVariable.tsUpdateCommitQCounter>vfCommand.counter)
                            {
                                vfValueReturn=-1;
                                vfTrans.commit=false;
                                vcError.fPutError(1303,"Transaction Manager Error : This query was done before Transaction No: "+vfVariable.tsUpdate+
                                " Commit: "+vpCommit+" '"+vfCommand.command+"' ROLLBACK READ this transaction : "+vfTransNo+", because tsNow < tsUpdate, will not be commited",true);
                            }
                            else if(vfTransNo<vfVariable.tsUpdate)
                            {
                                // set ts(read x) = max (ts(read x), ts(Ti))
                                vfVariable.tsRead=(vfVariable.tsRead>vfTransNo ? vfVariable.tsRead: vfTransNo);
                            }

                        }
                        else
                        if(vfCommand.po.command>IDDBConstants.COMMAND_SELECT)
                        {
                            vfStartTime=System.currentTimeMillis();
                            vfCounter=0;
                            while(((vfVariable.update.indexOf(vfCommand)>0)&&(vfCounter<vcDelay)))
                                vfCounter=System.currentTimeMillis()-vfStartTime;
                            if(vfCounter>=vcDelay)
                            {
                                vfValueReturn=-1;
                                vfTrans.commit=false;
                                vcError.fPutError(1304,"Transaction Manager Error : BLOCKED by query 'update: "+vfVariable.str+"' from Transaction No: "+((command_t)vfVariable.update.firstElement()).trans.transNo.intValue()+
                                    " "+vpCommit+" '"+vfCommand.command+"' ROLLBACK UPDATE this transaction: "+vfTransNo+", because will not be commited",true);
                            }
                            else if(vfTransNo<vfVariable.tsUpdate)
                            {
                                vfValueReturn=-1;
                                vfTrans.commit=false;
                                vcError.fPutError(1305,"Transaction Manager Error : "+vpCommit+" '"+vfCommand.command+"' ROLLBACK UPDATE this Transaction No: "+vfTransNo+", because tsNow < tsUpdate, will not be commited",true);
                            }
                            else if(vfTransNo<vfVariable.tsRead)
                            {
                                vfValueReturn=-1;
                                vfTrans.commit=false;
                                vcError.fPutError(1306,"Transaction Manager Error : "+vpCommit+" '"+vfCommand.command+"' ROLLBACK UPDATE this Transaction No: "+vfTransNo+", because tsNow < tsRead, will not be commited",true);
                            }
                            else
                            {
                                // set ts(update x=ts(T))
                                vfVariable.tsUpdate=vfTransNo;
                                vfVariable.tsUpdateCommitQCounter=vcQueryNo;

                                //lakukan update v|
                            }
                        }
                    }

                    // hapus elemen vfTrans, di queueTrans
                    // hapus elemen semua vfVariable-vfVariable di READ, UPDATE.
                    if(vfCommand.po.command>IDDBConstants.COMMAND_SELECT)
                        vfVariable.update.removeElement(vfCommand);
                }
                if(vcDebugLevel>=1)
                    System.out.println("vfTransNo ("+vfTrans.transNo.intValue()+"):"+((vfValueReturn==-1)||!vfTrans.commit?"not Commit":"Commit")+"vfTrans");
            }
            // lakukan update

            if(vfTrans.commit)
            {
                for (Enumeration vfEn = vfTrans.command.elements() ; vfEn.hasMoreElements() ;)
                {
                    command_t vfCommand= (command_t) vfEn.nextElement();

                    if(vfCommand.po.command>IDDBConstants.COMMAND_SELECT)
                    {
                        try
                        {
                            // operasi UPDATE, tanpa return ResultOut
                            vfRSOut=OptSch.executeQuery(vfCommand.po);
                		} catch(SQLException vfEx)
                		{
            	            vcError.fPutError(1307,"Transaction Manager Error : Encounter Error during TransactionManager Proccess, Exception: " + vfEx.toString(),true);
                		}
                	}
                }
            }

            // hapus elemen vfTrans, di queueTrans
            vfTrans.command.removeAllElements();
            vcQueueTrans.remove(vpConnectorNo);
        }
        return vfRSOut;
    }

    public ResultOut fPutQuery(Integer vpConnectorNo, boolean vpAutoCommit,int vpTransactionLevel, String vpQuery, String vpUser, int vpUserLevel) //throws SQLException
    {
        vcQueryNo++;

        parserout_t vfPO = getParser(vpQuery);
        vfPO.vcUser=vpUser;
        vfPO.vcUserLevel=vpUserLevel;
        vfPO.query=vpQuery;
        vfPO.queryno=vcQueryNo;

        command_t vfCommand=new command_t(vpConnectorNo,vpQuery,vfPO);
        trans_t vfTrans;
        variable_t vfVariable;
        int vfTransNo=0;
        int vfValueReturn;
        ResultOut vfRSOut=null;

        vfCommand.counter=vcQueryNo;

        String vfQuery=Tools.fReplaceString(vpQuery,"'","''");

        if(vcDebugLevel>=1)
            System.out.println(vfQuery);

        try
        {
            vcStatement.executeUpdate("insert into iddblog values('"+
            (new java.util.Date(System.currentTimeMillis())).toString()+"',"+vcLogCounter+", '"+vfQuery+"',"+vfPO.command+",'"+vpUser+"')");
     	    vcStatement.executeUpdate("update iddbtbcount set icounter = "+ ++vcLogCounter+" where itbname='log' and isiteid = '*'");

		} catch(SQLException vfEx)
		{
            vcError.fPutError(1308,"Transaction Manager Error : Encounter Error during write log, Exception: " + vfEx.toString(),true);
		}

        if((vcTransType==IDDBConstants.TRANS_SINGLE)&&(vcQueueTrans.size()>1))
        {
            vcError.fPutError(1309,"Transaction Manager Error : Transaction Single mode, wait for next transaction",true);
            return vfRSOut;
        }

        if((vfPO.command<IDDBConstants.COMMAND_CREATE_TABLE)&&!vpAutoCommit&&(vcTransType!=IDDBConstants.TRANS_NONE))
        {
            // cari trans, sesuai connectorNo, kalau tidak ada buat baru
            if(vcQueueTrans.containsKey(vpConnectorNo))
            {
                vfTrans=(trans_t) vcQueueTrans.get(vpConnectorNo);
                vfTransNo=vfTrans.transNo.intValue();
            }
            else
            {
                vfTransNo= vcTransNoCounter;
                vcQueueTrans.put(vpConnectorNo,vfTrans=new trans_t(new Integer(vcTransNoCounter++),vpConnectorNo));
            }

            try
            {

                if(vfPO.command==IDDBConstants.COMMAND_COMMIT)
                {
                    vfRSOut=fRemoveTransaction(vpConnectorNo,vpAutoCommit,vpTransactionLevel,"COMMIT");
                    vfRSOut = new ResultOut();
                }
                else if(vfPO.command==IDDBConstants.COMMAND_ROLLBACK)
                {
                    // hapus elemen semua vfVariable-vfVariable di READ, UPDATE.
                    for (Enumeration vfEn = vfTrans.command.elements() ; vfEn.hasMoreElements() ;)
                    {
                        vfCommand= (command_t) vfEn.nextElement();
                        for(Enumeration vfEn2 = vfCommand.po.table.elements() ; vfEn2.hasMoreElements() ;)
                        {
                            String vfTable;
                            tname_t vfTName= (tname_t) vfEn2.nextElement();
                            if(vfTName.cname!=null)
                    	        vfTable=vfTName.cname;
                    	    else
                    	        vfTable=vfTName.name;

                            if(vfCommand.po.command!=IDDBConstants.COMMAND_SELECT)
                                ((variable_t)vcQueueVariable.get(vfTable)).update.removeElement(vfCommand);
                        }
                    }

                    // hapus elemen vfTrans, di queueTrans
                    vfTrans.command.removeAllElements();
                    vcQueueTrans.remove(vpConnectorNo);

                    // queryUpdate("ROLLBACK");
                }
                else
                {
                    vfCommand.trans=vfTrans;

                    // masukkan command ke vfTrans
                    vfTrans.command.addElement(vfCommand);



                    // cari variable,sesuai string variable, kalau tidak ada buat baru
                    // JIKA VARIABLE LEBIH DARI SATU, ATAU KOSONG.
                    // SEMENTARA INI, SATU COMMAND, SATU VARIABLE
                    int vfVariableCount=0;
                    for(Enumeration ex = vfPO.table.elements() ; ex.hasMoreElements() ;vfVariableCount++)
                    {
                        String vfTable;
                        tname_t vfTName= (tname_t) ex.nextElement();
                        if( vfTName.cname!=null)
                	        vfTable= vfTName.cname;
                	    else
                	        vfTable= vfTName.name;

                        if(vcQueueVariable.containsKey(vfTable))
                        {
                            vfVariable=(variable_t) vcQueueVariable.get(vfTable);
                        }
                        else
                            vcQueueVariable.put(vfTable,vfVariable=new variable_t(vfTable));

                        // masukkan command dalam queue variable, sesuai tipe perintah READ/UPDATE
                        if(vfPO.command==IDDBConstants.COMMAND_SELECT)
                        {
                            // cek apakah ada update dari trans sebelumnya yang masih ada,
                            // kalau ya tunggu, sampai timeout
                            // ts(Ti)>ts(Tj)

                            if(vfTransNo<vfVariable.tsUpdate)
                            {
                                vfValueReturn=-1;
                                vfTrans.commit=false;
                                vcError.fPutError(1310,"Transaction Manager Error : Query: "+vfCommand.command+" ROLLBACK READ this Transaction No: "+vfTransNo+", because tsNow < tsUpdate, will not be commited",true);
                            }
                            else
                            {
                                // OPERASI READ
                                if(vfVariableCount==0)
                                    vfRSOut=OptSch.executeQuery(vfPO);
                            }
                        }
                        else if(vfPO.command>IDDBConstants.COMMAND_SELECT)
                        {
                            if(vfTransNo<vfVariable.tsUpdate)
                            {
                                vfValueReturn=-1;
                                vfTrans.commit=false;
                                vcError.fPutError(1311,"Transaction Manager Error : Query: "+vfCommand.command+" ROLLBACK UPDATE this Transaction No: "+vfTransNo+", because tsNow < tsUpdate, will not be commited",true);
                            }
                            else if(vfTransNo<vfVariable.tsRead)
                            {
                                vfValueReturn=-1;
                                vfTrans.commit=false;
                                vcError.fPutError(1312,"Transaction Manager Error : Query: "+vfCommand.command+" ROLLBACK UPDATE this Transaction No: "+vfTransNo+", because tsNow < tsRead, will not be commited",true);
                            }
                            else
                            {
                                vfVariable.update.putElement(vfCommand);
                                vfRSOut = new ResultOut();
                                vfRSOut.RSi=1;
                            }
                        }
                    }
                }
    		} catch(SQLException vfEx)
    		{
   	            vcError.fPutError(1307,"Transaction Manager Error : Encounter Error during TransactionManager Proccess, Exception: " + vfEx.toString(),true);
    		}
    	} else
    	{
    	    try {
    	        if(vfPO.command<IDDBConstants.COMMAND_CREATE_TABLE)
   	            {
                    for(Enumeration vfEn = vfPO.table.elements() ; vfEn.hasMoreElements() ;)
                    {
                        String vfTable;
                        tname_t vfTName= (tname_t) vfEn.nextElement();
                        if(vfTName.cname!=null)
                	        vfTable=vfTName.cname;
                	    else
                	        vfTable=vfTName.name;

                        if(vcQueueVariable.containsKey(vfTable))
                        {
                            vfVariable=(variable_t) vcQueueVariable.get(vfTable);
                        }
                        else
                            vcQueueVariable.put(vfTable,vfVariable=new variable_t(vfTable));

                        if(vfPO.command==IDDBConstants.COMMAND_SELECT)
                            vfVariable.tsRead=(vfVariable.tsRead>vcTransNoCounter ? vfVariable.tsRead: vcTransNoCounter);
                        else
                        {
                            vfVariable.tsUpdate=vcTransNoCounter++;
                            vfVariable.tsUpdateCommitQCounter=vcQueryNo;
                        }
                   }

    	        }
                vfRSOut=OptSch.executeQuery(vfPO);
    		} catch(SQLException vfEx)
    		{
   	            vcError.fPutError(1307,"Transaction Manager Error : Encounter Error during TransactionManager Proccess, Exception: " + vfEx.toString(),true);
    		}
    	}
        if(vfRSOut != null)
            vfRSOut.command=vfPO.command;
        return vfRSOut;
    }

    public void fExecuteQueryEnd()
    {
        OptSch.fExecuteQueryEnd();
    }
}